import torch

def argmax_with_threshold(tensor, threshold, default, dim=0):
    """
    Helper function to get the maximum indices of a tensor along a given dimension while using a
    threshold value. If all tensor elements are below the threshold value then return the value
    specified by the default.
    """
    max_indices = torch.argmax(tensor, dim=dim)
    max_values = torch.max(tensor, dim=dim).values

    below_threshold = max_values <= threshold
    max_indices[below_threshold] = default

    return max_indices


import string

from typing import Callable
from transformers.tokenization_utils_base import BatchEncoding

def convert_token_preds(
    tokenized_inputs: BatchEncoding,
    preds: list,
    tokenizer: Callable,
    skip_punc: bool = True,
):
    labels = []

    for i, label in enumerate(preds):
        word_ids = tokenized_inputs.word_ids(
            batch_index=i
        )  # Map tokens to their respective word.
        previous_word_idx = None
        label_tags = []

        for k, word_idx in enumerate(word_ids):
            token_str = tokenizer.convert_ids_to_tokens(
                int(tokenized_inputs.input_ids[i][k])
            )
            if word_idx is None:
                continue
            elif (
                word_idx != previous_word_idx and not skip_punc
            ):  # Only label the first token of a given word.
                label_tags.append(label[k])
            elif (
                word_idx != previous_word_idx
                and skip_punc
                and token_str not in string.punctuation
            ):
                # Remove BIO tags, keep only entity label
                label_tags.append(label[k].split("-")[-1])
            previous_word_idx = word_idx

        labels.append(label_tags)
    return labels


from hufr.models.ner import TokenClassificationTransformer
from hufr.constants import DEFAULT_REDACTION_MAP, DEFAULT_MODEL
from typing import Union

def redact_text(
    text: str,
    model: Union[TokenClassificationTransformer, None] = None,
    redaction_map: dict = DEFAULT_REDACTION_MAP,
    return_preds: bool = False,
):
    if model is None:
        model = TokenClassificationTransformer.from_pretrained(DEFAULT_MODEL)
        
    predictions = model.predict(text)
    text_lines = text.split()
    redacted_text_lines = []
    if any(pred in redaction_map for pred in predictions):
        for pred, text in zip(predictions, text_lines):
            redacted_text_lines.append(redaction_map.get(pred, text))
    else:
        redacted_text_lines = text_lines
    if return_preds:
        return " ".join(redacted_text_lines), predictions
    return " ".join(redacted_text_lines)